
%% Declearing and initializing parameters, variables and matrices
%Initialize
GeneratorNumber = 1;
UseNumericParameters = 2;
%SampleTime = 0.001;
% Linearization points

% Helping parameters
syms o

% Variables
N_x = 4; % Number of states
N_y = 3; % Number of measurements
N_u = 4; % Number of inputs
N_f = 2; % Number of faults (modeled)
N_constants = 10; % Number of constans 
N_diff = 1; % Number of differentiations

% x = [delta omega t_m i_f]'
syms x1 x2 x3 x4 

% y = [omega p q]' 
syms y1 y2 y3 
syms d_y1 d_y2 d_y3  
syms dd_y1 dd_y2 dd_y3

% u = [v_ll u v_f]'
syms u1 u2 u3 u4
syms d_u1 d_u2 d_u3 d_u4
syms dd_u1 dd_u2 dd_u3 dd_u4

% f = [f_u f_vf]'
syms f1 f2 
syms d_f1 d_f2 
syms dd_f1 dd_f2 

x = [x1; x2; x3; x4];

d_y = [d_y1; d_y2; d_y3];
dd_y = [dd_y1; dd_y2; dd_y3];

u = [u1; u2; u3; u4];
d_u = [d_u1; d_u2; d_u3; d_u4];
dd_u = [dd_u1; dd_u2; dd_u3; dd_u4];

f = [f1; f2];
d_f = [d_f1; d_f2];
dd_f = [dd_f1; dd_f2];


% Helping matrices for calculations. Must be the same size as y
temp_dy = [o;o;o];
temp_ddy = [o;o;o];

%% Defining equations
if UseNumericParameters == 0
    syms a b c d e g h k m n
    syms x_01 x_02 x_03 x_04 
    syms u_01 u_02 u_03 u_04 
    syms f_01 f_02 
    
    x_sym0 = [x_01;x_02;x_03;x_04];
    u_sym0 = [u_01;u_02;u_03;u_04];
    f_sym0 = [f_01;f_02];
    
elseif UseNumericParameters == 1
    Initialize
    a = omega_b;
    b = 1/(tau_f_G(GeneratorNumber)*r_f_G(GeneratorNumber));
    c = 1/(2*H_G(GeneratorNumber));
    d = D_fw_G(GeneratorNumber);
    e = 1/z_G(GeneratorNumber,2);
    g = r_f_G(GeneratorNumber);
    h = 1/tau_e_G(GeneratorNumber);
    k = (D_fw_G(GeneratorNumber)+1)/tau_e_G(GeneratorNumber);
    m = 1/tau_f_G(GeneratorNumber);
    n = z_G(GeneratorNumber,1)/z_G(GeneratorNumber,2)^2;
    
    syms a b c d e g h k m n
    syms x_01 x_02 x_03 x_04 
    syms u_01 u_02 u_03 u_04 
    syms f_01 f_02 
    
    x_sym0 = [x_01;x_02;x_03;x_04];
    u_sym0 = [u_01;u_02;u_03;u_04];
    f_sym0 = [f_01;f_02];
    
elseif UseNumericParameters == 2
    Initialize
    a = omega_b;
    b = 1/(tau_f_G(GeneratorNumber)*r_f_G(GeneratorNumber));
    c = 1/(2*H_G(GeneratorNumber));
    d = D_fw_G(GeneratorNumber);
    e = 1/z_G(GeneratorNumber,2);
    g = r_f_G(GeneratorNumber);
    h = 1/tau_e_G(GeneratorNumber);
    k = (D_fw_G(GeneratorNumber)+1)/tau_e_G(GeneratorNumber);
    m = 1/tau_f_G(GeneratorNumber);
    n = z_G(GeneratorNumber,1)/z_G(GeneratorNumber,2)^2;
    
    x_sym0 = x_0;
    u_sym0 = u_0;
    f_sym0 = f_0;
end
    
    
d_x = [o;o;o;o];
d_x(1) = a*(x2 - u(1));
d_x(2) = c*(x3 - d*x2 - e*x1/x2 - n*x1^2/(x2*u(2)^2) - n*(g*x2*x4-u(2))^2/(x2*u(2)^2));
%d_x(2) = c*(x3 - d*x2 - e*x1/x2); % Without electric loss
d_x(3) = -h*x3 + k*u(3) + k*f1;
d_x(4) = -m*x4 + b*u(4) + b*f2;

y = [o; o; o]; % y = f_y(x,u) 
y(1) = -y(1) + x(2);
y(2) = -y(2) + e*x(1);
y(3) = -y(3) + e*g*x(2)*x(4)-e*u(2);

%% Linearizing equations
Vector_x_u_f = [x;u;f];
Vector_x_u_f_linpoint = [x_sym0;u_sym0;f_sym0];

% Linearizing
jacobian_dx = jacobian(d_x,Vector_x_u_f);
LinMatrix_dx = subs(jacobian_dx,Vector_x_u_f,Vector_x_u_f_linpoint);
lin_d_x = simplify(LinMatrix_dx*Vector_x_u_f) % Linearized d_x

jacobian_y = jacobian(y,Vector_x_u_f);
LinMatrix_y = subs(jacobian_y,Vector_x_u_f,Vector_x_u_f_linpoint);
lin_y = simplify(LinMatrix_y*Vector_x_u_f) % Linearized y

% Defining the the matrices in the linearised state-space model
[combined_A_x, combined_b_x] = equationsToMatrix(lin_d_x,Vector_x_u_f);
[combined_A_y, combined_b_y] = equationsToMatrix(lin_y,Vector_x_u_f);

% lin_d_x = A*x+B*u+E_x*d+F_x*f
A = combined_A_x(1:N_x,1:N_x)
B = combined_A_x(1:N_x,N_x+1:N_x+N_u)
F_x = combined_A_x(1:N_x,N_x+N_u+1:N_x+N_u+N_f);
E_x = 0;
constTerm_x = -1*combined_b_x;
B_uf = combined_A_x(1:N_x,N_x+1:N_x+N_u+N_f);

% lin_y = C*x+D*u+E_y*d+F_y*f
C = combined_A_y(1:N_y,1:N_x)
D = combined_A_y(1:N_y,N_x+1:N_x+N_u)
F_y = combined_A_y(1:N_y,N_x+N_u+1:N_x+N_u+N_f);
E_y = 0;
constTerm_y = -1*combined_b_y;
D_uf = combined_A_y(1:N_y,N_x+1:N_x+N_u+N_f);

K = zeros(4,3);

lin_d_x
test_dx = simplify(A*x+B*u)
test_error_x = simplify(lin_d_x-F_x*f - test_dx)

lin_y
test_y = simplify(C*x+D*u)
test_error_y = simplify(lin_y - test_y)

temp_Du = D*u
temp_y_min_Cx = y-C*x


%% Testing system observability
if UseNumericParameters == 2
    A_num = double(A);
    C_num = double(C);
    ObsMatrix = obsv(A_num,C_num);
    RankObsMatrix = rank(ObsMatrix);
    SystemObservability = RankObsMatrix-N_x;
    if SystemObservability == 0
        disp('The system is observable')
    else
        disp('The system is not observable')
    end
else
    disp('Can not calculate observability from symboloc matrices')
end











