%% Declearing and initializing parameters, variables and matrices

% Linearization points
syms x_01 x_02 x_03 x_04
syms u_01 u_02 u_03
syms f_01 f_02
x_0 = [x_01; x_02; x_03; x_04];
u_0 = [u_01; u_02; u_03;];
f_0 = [f_01; f_02];

% Helping parameters
syms o

% Variables
N_x = 4; % Number of states
N_y = 3; % Number of measurements
N_u = 3; % Number of inputs
N_f = 2; % Number of faults (modeled)
N_constants = 10; % Number of constans 
N_diff = 1; % Number of differentiations

% x = [delta omega t_m i_f]'
syms x1 x2 x3 x4 

% y = [omega p q]' 
syms y1 y2 y3 
syms d_y1 d_y2 d_y3  
syms dd_y1 dd_y2 dd_y3

% u = [v_ll u v_f]'
syms u1 u2 u3  
syms d_u1 d_u2 d_u3
syms dd_u1 dd_u2 dd_u3

% f = [f_u f_vf]'
syms f1 f2 
syms d_f1 d_f2 
syms dd_f1 dd_f2 

x = [x1; x2; x3; x4];

y = [y1; y2; y3];
d_y = [d_y1; d_y2; d_y3];
dd_y = [dd_y1; dd_y2; dd_y3];

u = [u1; u2; u3];
d_u = [d_u1; d_u2; d_u3];
dd_u = [dd_u1; dd_u2; dd_u3];

f = [f1; f2];
d_f = [d_f1; d_f2];
dd_f = [dd_f1; dd_f2];

% Helping matrices for calculations. Must be the same size as y
temp_dy = [o;o;o];
temp_ddy = [o;o;o];

%% Defining equations
syms a b c d e g h k m n

d_x = [o;o;o;o];
d_x(1) = a*x2;
d_x(2) = c*(x3 - d*x2 - e*x1/x2 - n*x1^2/(x2*u(1)^2) - n*(g*x2*x4-u(1))^2/(x2*u(1)^2));
%d_x(2) = c*(x3 - d*x2 - e*x1/x2); % Without electric loss
d_x(3) = -h*x3 + k*u(2) + k*f1;
d_x(4) = -m*x4 + b*u(3) + b*f2;

temp_y = [o; o; o]; % y = f_y(x,u) => temp_y = y-f_y(x,u) = 0
temp_y(1) = -y(1) + x(2);
temp_y(2) = -y(2) + e*x(1);
temp_y(3) = -y(3) + e*g*x(2)*x(4)-e*u(1);
y_func = temp_y+y; % y = f_y(x,u)

% Defining vectors to enable conversion from matlab to latex symbols 
syms delta_i omega_i t_mi i_fi % x parameters
syms delta_0i omega_0i t_m0i i_f0i
syms omega_i p_i q_i d_omega_i d_p_i d_q_i dd_omega_i dd_p_i dd_q_i % y parameters
syms v_ll u_i v_fi d_v_ll d_u_i d_v_fi dd_v_ll dd_u_i dd_v_fi % u parameters
syms v_ll0 u_0i v_f0i
syms f_u f_vf d_f_u d_f_vf dd_f_u dd_f_vf % f parameters
syms f_u0i f_vf0i
syms omega_b tau_fi r_fi H_i D_fwi x_si tau_ei r_si % constants parameters

parameter_symbols_x = [x1; x2; x3; x4];
parameters_x = [delta_i; omega_i; t_mi; i_fi];

parameter_symbols_x0 = [x_01; x_02; x_03; x_04];
parameters_x0 = [delta_0i; omega_0i; t_m0i; i_f0i];

parameter_symbols_y = [y1; y2; y3; d_y1; d_y2; d_y3; dd_y1; dd_y2; dd_y3];
parameters_y = [omega_i; p_i; q_i; d_omega_i; d_p_i; d_q_i; dd_omega_i; dd_p_i; dd_q_i];

parameter_symbols_u = [u1; u2; u3; d_u1; d_u2; d_u3; dd_u1; dd_u2; dd_u3];
parameters_u = [v_ll; u_i; v_fi; d_v_ll; d_u_i; d_v_fi; dd_v_ll; dd_u_i; dd_v_fi];

parameter_symbols_u0 = [u_01; u_02; u_03];
parameters_u0 = [v_ll0; u_0i; v_f0i];

parameter_symbols_f = [f1; f2; d_f1; d_f2; dd_f1; dd_f2];
parameters_f = [f_u; f_vf; d_f_u; d_f_vf; dd_f_u; dd_f_vf];

parameter_symbols_f0 = [f_01; f_02];
parameters_f0 = [f_u0i; f_vf0i];

parameter_symbols_constants = [a; b; c; d; e; g; h; k; m; n];
parameters_constants = [omega_b; 1/(tau_fi*r_fi); 1/(2*H_i); D_fwi;1/x_si; r_fi; 1/tau_ei; (D_fwi+1)/tau_ei; 1/tau_fi; r_si/x_si^2];

parameter_symbols = [parameter_symbols_x; parameter_symbols_x0; parameter_symbols_y; parameter_symbols_u; parameter_symbols_u0; parameter_symbols_f; parameter_symbols_f0; parameter_symbols_constants];
parameters = [parameters_x; parameters_x0; parameters_y; parameters_u; parameters_u0; parameters_f; parameters_f0; parameters_constants];

N_parameter_symbols = size(parameter_symbols,1);
N_parameters = size(parameters,1);

symbols_and_prameters = [parameter_symbols parameters];


%% Linearizing equations
Vector_x_u_f = [x;u;f];
Vector_x_u_f_linpoint = [x_0;u_0;f_0];

% Linearizing
jacobian_dx = jacobian(d_x,Vector_x_u_f);
LinMatrix_dx = subs(jacobian_dx,Vector_x_u_f,Vector_x_u_f_linpoint);
lin_d_x = LinMatrix_dx*Vector_x_u_f; % Linearized d_x

jacobian_y = jacobian(y_func,Vector_x_u_f);
LinMatrix_y = subs(jacobian_y,Vector_x_u_f,Vector_x_u_f_linpoint);
lin_y = LinMatrix_y*Vector_x_u_f; % Linearized y

% Defining the the matrices in the linearised state-space model
[combined_A_x, combined_b_x] = equationsToMatrix(lin_d_x,Vector_x_u_f);
[combined_A_y, combined_b_y] = equationsToMatrix(lin_y,Vector_x_u_f);

% lin_d_x = A*x+B*u+E_x*d+F_x*f
A = combined_A_x(1:N_x,1:N_x);
B = combined_A_x(1:N_x,N_x+1:N_x+N_u);
F_x = combined_A_x(1:N_x,N_x+N_u+1:N_x+N_u+N_f);
E_x = 0;
constTerm_x = -1*combined_b_x;

% lin_y = C*x+D*u+E_y*d+F_y*f
C = combined_A_y(1:N_y,1:N_x);
D = combined_A_y(1:N_y,N_x+1:N_x+N_u);
F_y = combined_A_y(1:N_y,N_x+N_u+1:N_x+N_u+N_f);
E_y = 0;
constTerm_y = -1*combined_b_y;

%% Testing system observability

ObsMatrix = obsv(A,C);
RankObsMatrix = rank(ObsMatrix);
SystemObservability = RankObsMatrix-N_x

%% Replacing variables
ReplaceVariables = 0;
if ReplaceVariables == 1
    lin_d_x_par = simplify(subs(lin_d_x,parameter_symbols(1),parameters(1)));
    lin_y_par = simplify(subs(lin_y,parameter_symbols(1),parameters(1)));
    A_par = simplify(subs(A,parameter_symbols(1),parameters(1)));
    B_par = simplify(subs(B,parameter_symbols(1),parameters(1)));
    F_x_par = simplify(subs(F_x,parameter_symbols(1),parameters(1)));
    C_par = simplify(subs(C,parameter_symbols(1),parameters(1)));
    D_par = simplify(subs(D,parameter_symbols(1),parameters(1)));
    F_y_par = simplify(subs(F_y,parameter_symbols(1),parameters(1)));
    for i = 1:(N_parameters)
        lin_d_x_par = simplify(subs(lin_d_x_par,parameter_symbols(i),parameters(i)));
        lin_y_par = simplify(subs(lin_y_par,parameter_symbols(i),parameters(i)));
        A_par = simplify(subs(A_par,parameter_symbols(i),parameters(i)));
        B_par = simplify(subs(B_par,parameter_symbols(i),parameters(i)));
        F_x_par = simplify(subs(F_x_par,parameter_symbols(i),parameters(i)));
        C_par = simplify(subs(C_par,parameter_symbols(i),parameters(i)));
        D_par = simplify(subs(D_par,parameter_symbols(i),parameters(i)));
        F_y_par = simplify(subs(F_y_par,parameter_symbols(i),parameters(i)));
    end

    %lin_d_x_par
    %lin_y_par
    % A_par
    % A_par_1= A_par(1,:)
    % A_par_2= A_par(2,:)
    % A_par_3= A_par(3,:)
    % A_par_4= A_par(4,:)
    % B_par
    % F_x_par
    % C_par 
    % D_par
    F_y_par 
end
    
    