
%% Declearing and initializing parameters, variables and matrices

% Linearization points
syms x_01 x_02 x_03 x_04
syms u_01 u_02 u_03 u_04
syms f_01 f_02
x_0 = [x_01; x_02; x_03; x_04];
u_0 = [u_01; u_02; u_03; u_04];
f_0 = [f_01; f_02];

% Helping parameters
syms o

% Variables
N_x = 4; % Number of states
N_y = 3; % Number of measurements
N_u = 4; % Number of inputs
N_f = 2; % Number of faults (modeled)
N_constants = 10; % Number of constans 
N_diff = 1; % Number of differentiations

% x = [delta omega t_m i_f]'
syms x1 x2 x3 x4 

% y = [omega p q]' 
syms y1 y2 y3 
syms d_y1 d_y2 d_y3  
syms dd_y1 dd_y2 dd_y3

% u = [omega_0 v_ll u v_f]'
syms u1 u2 u3 u4
syms d_u1 d_u2 d_u3 d_u4
syms dd_u1 dd_u2 dd_u3 dd_u4

% f = [f_u f_vf]'
syms f1 f2 
syms d_f1 d_f2 
syms dd_f1 dd_f2 

x = [x1; x2; x3; x4];

y = [y1; y2; y3];
d_y = [d_y1; d_y2; d_y3];
dd_y = [dd_y1; dd_y2; dd_y3];

u = [u1; u2; u3; u4];
d_u = [d_u1; d_u2; d_u3; d_u4];
dd_u = [dd_u1; dd_u2; dd_u3; dd_u4];

f = [f1; f2];
d_f = [d_f1; d_f2];
dd_f = [dd_f1; dd_f2];

% Helping matrices for calculations. Must be the same size as y
temp_dy = [o;o;o];
temp_ddy = [o;o;o];

%% Defining equations
syms a b c d e g h k m n

d_x = [o;o;o;o];
d_x(1) = a*(x2-u(1));
d_x(2) = c*(x3 - d*x2 - e*x1/x2 - n*x1^2/(x2*u(2)^2) - n*(g*x2*x4-u(2))^2/(x2*u(2)^2));
%d_x(2) = c*(x3 - d*x2 - e*x1/x2); % Without electric loss
d_x(3) = -h*x3 + k*u(3) + k*f1;
d_x(4) = -m*x4 + b*u(4) + b*f2;

temp_y = [o; o; o]; % y = f_y(x,u) => temp_y = y-f_y(x,u) = 0
temp_y(1) = -y(1) + x(2);
temp_y(2) = -y(2) + e*x(1);
temp_y(3) = -y(3) + e*g*x(2)*x(4)-e*u(2);
y_func = temp_y+y; % y = f_y(x,u)

% Defining vectors to enable conversion from matlab to latex symbols 
syms delta_i omega_i t_mi i_fi % x parameters
syms omega_i p_i q_i d_omega_i d_p_i d_q_i dd_omega_i dd_p_i dd_q_i % y parameters
syms omega_0 v_ll u_i v_fi d_omega_0 d_v_ll d_u_i d_v_fi dd_omega_0 dd_v_ll dd_u_i dd_v_fi % u parameters
syms f_u f_vf d_f_u d_f_vf dd_f_u dd_f_vf % f parameters
syms omega_b tau_f r_f H D_fw x_s tau_e r_s % constants parameters

parameter_symbols_x = [x1; x2; x3; x4];
parameters_x = [delta_i; omega_i; t_mi; i_fi];

parameter_symbols_y = [y1; y2; y3; d_y1; d_y2; d_y3; dd_y1; dd_y2; dd_y3];
parameters_y = [omega_i; p_i; q_i; d_omega_i; d_p_i; d_q_i; dd_omega_i; dd_p_i; dd_q_i];

parameter_symbols_u = [u1; u2; u3; u4; d_u1; d_u2; d_u3; d_u4; dd_u1; dd_u2; dd_u3; dd_u4];
parameters_u = [omega_0; v_ll; u_i; v_fi; d_omega_0; d_v_ll; d_u_i; d_v_fi; dd_omega_0; dd_v_ll; dd_u_i; dd_v_fi];

parameter_symbols_f = [f1; f2; d_f1; d_f2; dd_f1; dd_f2];
parameters_f = [f_u; f_vf; d_f_u; d_f_vf; dd_f_u; dd_f_vf];

parameter_symbols_constants = [a; b; c; d; e; g; h; k; m; n];
parameters_constants = [omega_b; 1/(tau_f*r_f); 1/(2*H); D_fw;1/x_s; r_f; 1/tau_e; (D_fw+1)/tau_e; 1/tau_f; r_s/x_s^2];

parameter_symbols = [parameter_symbols_y; parameter_symbols_u; parameter_symbols_f; parameter_symbols_constants];
parameters = [parameters_y; parameters_u; parameters_f; parameters_constants];

N_parameter_symbols = size(parameter_symbols,1);
N_parameters = size(parameters,1);

symbols_and_prameters = [parameter_symbols parameters];


%% Linearizing equations
Vector_x_u_f = [x;u;f];
Vector_x_u_f_linpoint = [x_0;u_0;f_0];

% Linearizing
jacobian_dx = jacobian(d_x,Vector_x_u_f);
LinMatrix_dx = subs(jacobian_dx,Vector_x_u_f,Vector_x_u_f_linpoint);
lin_d_x = LinMatrix_dx*Vector_x_u_f % Linearized d_x

jacobian_y = jacobian(y_func,Vector_x_u_f);
LinMatrix_y = subs(jacobian_y,Vector_x_u_f,Vector_x_u_f_linpoint);
lin_y = LinMatrix_y*Vector_x_u_f % Linearized y

% Defining the the matrices in the linearised state-space model
[combined_A_x, combined_b_x] = equationsToMatrix(lin_d_x,Vector_x_u_f);
[combined_A_y, combined_b_y] = equationsToMatrix(lin_y,Vector_x_u_f);

% lin_d_x = A*x+B*u+E_x*d+F_x*f
A = combined_A_x(1:N_x,1:N_x)
B = combined_A_x(1:N_x,N_x+1:N_x+N_u)
F_x = combined_A_x(1:N_x,N_x+N_u+1:N_x+N_u+N_f)
E_x = 0;
constTerm_x = -1*combined_b_x

% lin_y = C*x+D*u+E_y*d+F_y*f
C = combined_A_y(1:N_y,1:N_x)
D = combined_A_y(1:N_y,N_x+1:N_x+N_u)
F_y = combined_A_y(1:N_y,N_x+N_u+1:N_x+N_u+N_f)
E_y = 0;
constTerm_y = -1*combined_b_y
    
%% Computing the matrices O, T_dq, T_uq for the linearized system
q = N_diff;
p = N_y;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 1 differentiation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if q == 0
    O = C
    T_uq = D
    T_fq = F_y;
    T_dq = zeros(size(T_uq,1),size(T_uq,2));
    
    O_T_dq =  [O T_dq]
    
    n_r = double((q+1)*p-rank(O_T_dq))
    W_symbolic = sym('w', [n_r (q+1)*p])
    
end
    
if q == 1
    O = [C;C*A];
    
    o_matrix = zeros(size(D,1),size(D,2));
    T_uq = [D o_matrix;C*B D]
    
    o_matrix = zeros(size(F_y,1),size(F_y,2));
    T_fq = [F_y o_matrix;C*F_x F_y];
    
    if E_x == 0 && E_y == 0
        T_dq = zeros(size(T_uq,1),size(T_uq,2));
    else
        o_matrix = zeros(size(E_y,1),size(E_y,2));
        T_dq = [E_y o_matrix;C*E_x E_y];
    end
    
    % Calculating W
    O_T_dq = [O T_dq]
    n_r = double((q+1)*p-rank(O_T_dq))
    W_symbolic = sym('w', [n_r (q+1)*p])

    W_symbolic_vector =  sym('o', [n_r*(q+1)*p, 1]);
    for i = 0:q
        counter1 = i*(q+1)*p+1;
        counter2 = (i+1)*(q+1)*p;
        W_symbolic_vector(counter1:counter2, 1) = transpose(W_symbolic(i+1,:))
    end
    W_symbolic_vector;

    W_O_T_dq = W_symbolic*[O T_dq];
    O_T_dq_newstruct = equationsToMatrix(W_O_T_dq,W_symbolic_vector);
    nulspace_O_T_dq_newstruct = null(O_T_dq_newstruct)

    size_W = [n_r (q+1)*p];
    N_W = size(nulspace_O_T_dq_newstruct,2);
    W_matrix = sym('o', [N_W*n_r (q+1)*p]);
    for i = 0:N_W-1
        counter1 = i*n_r+1;
        counter2 = (i+1)*n_r;
        counter3 = (q+1)*p;
        W_matrix(counter1:counter2,:) = [transpose(nulspace_O_T_dq_newstruct(1:counter3,i+1)); transpose(nulspace_O_T_dq_newstruct(counter3+1:2*counter3,i+1))];    
    end
    N_W
    W_matrix

    % test1 = W_matrix(1:2,:)*O_T_dq
    % test2 = W_matrix(3:4,:)*O_T_dq
    % test3 = W_matrix(5:6,:)*O_T_dq
    % test4 = W_matrix(7:8,:)*O_T_dq
    test1_u = W_matrix(1:2,:)*T_uq
    test2_u = W_matrix(3:4,:)*T_uq
    test3_u = W_matrix(5:6,:)*T_uq
    test4_u = W_matrix(7:8,:)*T_uq

    T_fq
    test1_f = W_matrix(1:2,:)*T_fq
    test2_f = W_matrix(3:4,:)*T_fq
    test3_f = W_matrix(5:6,:)*T_fq
    test4_f = W_matrix(7:8,:)*T_fq
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 2 differentiations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
elseif q == 2
    O = [C;C*A;C*A^2];
    
    o_matrix = zeros(size(D,1),size(D,2));
    T_uq = [D o_matrix o_matrix;C*B D o_matrix;C*A*B C*B D];
    
    o_matrix = zeros(size(F_y,1),size(F_y,2));
    T_fq = [F_y o_matrix o_matrix;C*F_x F_y o_matrix;C*A*F_x C*F_x F_y];
    
    if E_x == 0 && E_y == 0
        T_dq = zeros(size(T_uq,1),size(T_uq,2));
    else
        o_matrix = zeros(size(E_y,1),size(E_y,2));
        T_dq = [E_y o_matrix o_matrix;C*E_x E_y o_matrix;C*A*E_x C*E_x E_y];
    end

    % Calculating W
    O_T_dq =  [O T_dq];
    n_r = double((q+1)*p-rank(O_T_dq));
    W_symbolic = sym('w', [n_r (q+1)*p]);

    W_symbolic_vector =  sym('o', [n_r*(q+1)*p, 1]);
    for i = 0:q
        counter1 = i*(q+1)*p+1;
        counter2 = (i+1)*(q+1)*p;
        W_symbolic_vector(counter1:counter2, 1) = transpose(W_symbolic(i+1,:));
    end
    W_symbolic_vector;

    W_O_T_dq = W_symbolic*[O T_dq];
    O_T_dq_newstruct = equationsToMatrix(W_O_T_dq,W_symbolic_vector);
    nulspace_O_T_dq_newstruct = null(O_T_dq_newstruct);

    size_W = [n_r (q+1)*p];
    N_W = size(nulspace_O_T_dq_newstruct,2);
    W_matrix = sym('o', [N_W*n_r (q+1)*p]);
    for i = 0:N_W-1
        counter1 = i*n_r+1;
        counter2 = (i+1)*n_r;
        counter3 = (q+1)*p;
        temp1 = transpose(nulspace_O_T_dq_newstruct(1:counter3,i+1));
        temp2 = transpose(nulspace_O_T_dq_newstruct(counter3+1:2*counter3,i+1));
        temp3 = transpose(nulspace_O_T_dq_newstruct(2*counter3+1:3*counter3,i+1));
        temp4 = transpose(nulspace_O_T_dq_newstruct(3*counter3+1:4*counter3,i+1));
        temp5 = transpose(nulspace_O_T_dq_newstruct(4*counter3+1:5*counter3,i+1));
        W_matrix(counter1:counter2,:) = [temp1;temp2;temp3;temp4;temp5];    
    end
    N_W;
    W_matrix;
    size_W_matrix = size(W_matrix)
   
    W_matrix_valid = sym('o', [1 (q+1)*p]);
    i0 = 1;
    for i = n_r:n_r:size(W_matrix,1)       
        test_O_T_dq = W_matrix(i0:i,:)*O_T_dq ;
        test_u = W_matrix(i0:i,:)*T_uq;
        test_f = W_matrix(i0:i,:)*T_fq;
        
        sum_W_O_T_dq = 0;
        sum_W_T_uq = 0;
        sum_W_T_fq = 0;
        for i2 = 1:size(test_O_T_dq,1)
            for i3 = 1:size(test_O_T_dq,2)
                if test_O_T_dq(i2,i3) ~= 0
                    sum_W_O_T_dq = sum_W_O_T_dq+1;
                end
            end
        end
        for i2 = 1:size(test_u,1)
            for i3 = 1:size(test_u,2)
                if test_u(i2,i3) ~= 0
                    sum_W_T_uq = sum_W_T_uq+1;
                end
            end
        end
        for i2 = 1:size(test_f,1)
            for i3 = 1:size(test_f,2)
                if test_f(i2,i3)~= 0
                    sum_W_T_fq = sum_W_T_fq+1;
                end
            end
        end
                
        % r = W*y_q - W*T_uq*u_q = W*T_fq*f_q ([O T_dq] = 0)
        if sum_W_O_T_dq == 0 && sum_W_T_fq ~= 0 && sum_W_T_uq~= 0
            W_matrix_valid = [W_matrix_valid; W_matrix(i0:i,:)];
        end
        i0 = i0+5;
    end
    W_matrix_valid(1,:) = []
    
      
	test1_u = W_matrix_valid(1:5,:)*T_uq
    test2_u = W_matrix_valid(6:10,:)*T_uq
    test3_u = W_matrix_valid(11:15,:)*T_uq
    
	test1_f = W_matrix_valid(1:5,:)*T_fq
    test2_f = W_matrix_valid(6:10,:)*T_fq
    test3_f = W_matrix_valid(11:15,:)*T_fq
end









