close all
clear all

%% Declearing and initializing parameters, variables and matrices
% Helping parameters
syms o

N_x = 4; % Number of states
N_y = 3; % Number of measurements
N_u = 4; % Number of inputs
N_f = 2; % Number of faults (modeled)
N_constants = 10; % Number of constans 
N_diff = 2; % Number of differentiations

% Variables
syms x1 x2 x3 x4 % [delta omega t_m i_f]'
syms y1 y2 y3 % [omega p q]' 
syms d_y1 d_y2 d_y3  
syms dd_y1 dd_y2 dd_y3
syms u1 u2 u3 u4 % [omega_0 v_ll u v_f]'
syms d_u1 d_u2 d_u3 d_u4
syms dd_u1 dd_u2 dd_u3 dd_u4
syms f1 f2 % [f_u f_vf]'
syms d_f1 d_f2 
syms dd_f1 dd_f2 

x = [x1; x2; x3; x4];
f = [f1; f2];
d_f = [d_f1; d_f2];
dd_f = [dd_f1; dd_f2];
y = [y1; y2; y3];
d_y = [d_y1; d_y2; d_y3];
dd_y = [dd_y1; dd_y2; dd_y3];
u = [u1; u2; u3; u4];
d_u = [d_u1; d_u2; d_u3; d_u4];
dd_u = [dd_u1; dd_u2; dd_u3; dd_u4];

temp_dy = [o;o;o];
temp_ddy = [o;o;o];

%% Defining equations
syms a b c d e g h k m n
d_x = [o;o;o;o];
d_x(1) = a*(x2-u(1));
d_x(2) = c*(x3 - d*x2 - e*x1/x2 - n*x1^2/(x2*u(2)^2) - n*(g*x2*x4-u(2))^2/(x2*u(2)^2));
%d_x(2) = c*(x3 - d*x2 - e*x1/x2); % Without electric loss
d_x(3) = -h*x3 + k*u(3) + k*f1;
d_x(4) = -m*x4 + b*u(4) + b*f2;

temp_y = [o; o; o];
temp_y(1) = -y(1) + x(2);
temp_y(2) = -y(2) + e*x(1);
temp_y(3) = -y(3) + e*g*x(2)*x(4)-e*u(2);

% Defining vectors to enable conversion from matlab to latex symbols 
% (x not included)
syms omega_i p_i q_i d_omega_i d_p_i d_q_i dd_omega_i dd_p_i dd_q_i %y parameters
syms omega_0 v_ll u_i v_fi d_omega_0 d_v_ll d_u_i d_v_fi dd_omega_0 dd_v_ll dd_u_i dd_v_fi %u parameters
syms f_u f_vf d_f_u d_f_vf dd_f_u dd_f_vf % f parameters
syms omega_b tau_f r_f H D_fw x_s tau_e r_s % constants parameters

parameter_symbols_y = [y1; y2; y3; d_y1; d_y2; d_y3; dd_y1; dd_y2; dd_y3];
parameters_y = [omega_i; p_i; q_i; d_omega_i; d_p_i; d_q_i; dd_omega_i; dd_p_i; dd_q_i];

parameter_symbols_u = [u1; u2; u3; u4; d_u1; d_u2; d_u3; d_u4; dd_u1; dd_u2; dd_u3; dd_u4];
parameters_u = [omega_0; v_ll; u_i; v_fi; d_omega_0; d_v_ll; d_u_i; d_v_fi; dd_omega_0; dd_v_ll; dd_u_i; dd_v_fi];

parameter_symbols_f = [f1; f2; d_f1; d_f2; dd_f1; dd_f2];
parameters_f = [f_u; f_vf; d_f_u; d_f_vf; dd_f_u; dd_f_vf];

parameter_symbols_constants = [a; b; c; d; e; g; h; k; m; n];
parameters_constants = [omega_b; 1/(tau_f*r_f); 1/(2*H); D_fw;1/x_s; r_f; 1/tau_e; (D_fw+1)/tau_e; 1/tau_f; r_s/x_s^2];

parameter_symbols = [parameter_symbols_y; parameter_symbols_u; parameter_symbols_f; parameter_symbols_constants];
parameters = [parameters_y; parameters_u; parameters_f; parameters_constants];

N_parameter_symbols = size(parameter_symbols,1);
N_parameters = size(parameters,1);

symbols_and_prameters = [parameter_symbols parameters];
%% Step 1: First derivation
for i1 = 1:N_y
    temp1 = 0;
    temp2 = 0;
    temp3 = 0;
    for i2 = 1:N_x
        temp1 = temp1 + diff(temp_y(i1),x(i2))*d_x(i2);
    end
    for i2 = 1:N_u
        temp2 = temp2 + diff(temp_y(i1),u(i2))*d_u(i2);
    end
    for i2 = 1:N_f
        temp3 = temp3 + diff(temp_y(i1),f(i2))*d_f(i2);
    end
    temp_dy(i1) = -d_y(i1)+temp1+temp2+temp3;
end

%% Step 2: Second derivation
for i1 = 1:N_y
    temp1 = 0;
    temp2 = 0;
    temp3 = 0;
    temp4 = 0;
    temp5 = 0;
    for i2 = 1:N_x
        temp1 = temp1 + diff(temp_dy(i1),x(i2))*d_x(i2);
    end
    for i2 = 1:N_u
        temp2 = temp2 + diff(temp_dy(i1),u(i2))*d_u(i2);
        temp3 = temp3 + diff(temp_dy(i1),d_u(i2))*dd_u(i2);
    end
    for i2 = 1:N_f
        temp4 = temp4 + diff(temp_dy(i1),f(i2))*d_f(i2);
        temp5 = temp5 + diff(temp_dy(i1),d_f(i2))*dd_f(i2);
    end
    temp_ddy(i1) = -dd_y(i1)+temp1+temp2+temp3+temp4+temp5; % = 0
end
%% Calculate result
y = temp_y + y
d_y = temp_dy + d_y
dd_y = temp_ddy + dd_y

% Solution 
S = solve(temp_y(1),temp_y(2),temp_y(3),temp_dy(1),x1,x2,x3,x4);
sol1_x1 = simplify(S.x1)
sol1_x2 = simplify(S.x2)
sol1_x3 = simplify(S.x3)
sol1_x4 = simplify(S.x4)

temp1_dy = subs(temp_dy,x1,sol1_x1);
temp2_dy = subs(temp1_dy,x2,sol1_x2);
temp3_dy = subs(temp2_dy,x3,sol1_x3);
residual1_dy = simplify(subs(temp3_dy,x4,sol1_x4))

temp1_ddy = subs(temp_ddy,x1,sol1_x1); 
temp2_ddy = subs(temp1_ddy,x2,sol1_x2); 
temp3_ddy = subs(temp2_ddy,x3,sol1_x3); 
residual1_ddy = simplify(subs(temp3_ddy,x4,sol1_x4))

%% Replacing variables % 

residual1_dy_par = simplify(subs(residual1_dy,parameter_symbols(1),parameters(1)));
residual1_ddy_par = simplify(subs(residual1_ddy,parameter_symbols(1),parameters(1)));
for i = 1:(N_parameters)
    residual1_dy_par = simplify(subs(residual1_dy_par,parameter_symbols(i),parameters(i)));
    residual1_ddy_par = simplify(subs(residual1_ddy_par,parameter_symbols(i),parameters(i)));
end
residual1_par_d_omega = latex(residual1_dy_par(1))
residual1_par_d_p = latex(residual1_dy_par(2))
residual1_par_d_q = latex(residual1_dy_par(3))
residual1_par_dd_omega = latex(residual1_ddy_par(1));
residual1_par_dd_p = latex(residual1_ddy_par(2));
residual1_par_dd_q = latex(residual1_ddy_par(3));


%% Generating matrices showing variable dependencies

Matrix_variable_dependence_residual1_dy = zeros(N_y,N_parameters);
Matrix_variable_dependence_residual1_ddy = zeros(N_y,N_parameters);
for i1 = 1:N_y
     temp_dy = symvar(residual1_dy(i1));
     temp_ddy = symvar(residual1_ddy(i1));
    for i2 = 1:N_parameters
     for i3 = 1:size(temp_dy,2)
        if isempty(temp_dy) == 1
            isempty_dd = 1;
        elseif temp_dy(i3) == parameter_symbols(i2) 
            Matrix_variable_dependence_residual1_dy(i1,i2) = 1;
        end 
     end
     for i3 = 1:size(temp_ddy,2)
        if isempty(temp_ddy) == 1
            isempty_dd = 1;
        elseif temp_ddy(i3) == parameter_symbols(i2)  
            Matrix_variable_dependence_residual1_ddy(i1,i2) = 1;
        end
     end
    end
end
Matrix_variable_dependence_residual1_dy;
Matrix_variable_dependence_residual1_ddy;

%parameters = [parameters_y; parameters_u; parameters_f; parameters_constants];
Dependence_dy_y = Matrix_variable_dependence_residual1_dy(1:N_y, 1:(N_diff+1)*N_y)
Dependence_dy_u = Matrix_variable_dependence_residual1_dy(1:N_y, (N_diff+1)*N_y+1:(N_diff+1)*N_y+(N_diff+1)*N_u)
Dependence_dy_f = Matrix_variable_dependence_residual1_dy(1:N_y, (N_diff+1)*N_y+(N_diff+1)*N_u+1:(N_diff+1)*N_y+(N_diff+1)*N_u+(N_diff+1)*N_f)
Dependence_dy_constants = Matrix_variable_dependence_residual1_dy(1:N_y, (N_diff+1)*N_y+(N_diff+1)*N_u+(N_diff+1)*N_f+1:(N_diff+1)*N_y+(N_diff+1)*N_u+(N_diff+1)*N_f+N_constants) 

Dependence_ddy_y = Matrix_variable_dependence_residual1_ddy(1:N_y, 1:(N_diff+1)*N_y)
Dependence_ddy_u = Matrix_variable_dependence_residual1_ddy(1:N_y, (N_diff+1)*N_y+1:(N_diff+1)*N_y+(N_diff+1)*N_u)
Dependence_ddy_f = Matrix_variable_dependence_residual1_ddy(1:N_y, (N_diff+1)*N_y+(N_diff+1)*N_u+1:(N_diff+1)*N_y+(N_diff+1)*N_u+(N_diff+1)*N_f)
Dependence_ddy_constants = Matrix_variable_dependence_residual1_ddy(1:N_y, (N_diff+1)*N_y+(N_diff+1)*N_u+(N_diff+1)*N_f+1:(N_diff+1)*N_y+(N_diff+1)*N_u+(N_diff+1)*N_f+N_constants) 



