%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%% Plotting
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%--------------------------------------------------------------------------
%---- Usage ---------------------------------------------------------------
%--------------------------------------------------------------------------
%   Plots simulated values from SimulationPlantModel
%--------------------------------------------------------------------------
%---- Inputs --------------------------------------------------------------
%--------------------------------------------------------------------------
%   No input
%--------------------------------------------------------------------------
%---- Outputs -------------------------------------------------------------
%--------------------------------------------------------------------------
%   Variable name       Dimension   Description  
%
%--------------------------------------------------------------------------
% Written by: Ida Bohne Krogseth
% Last Modified: 23/04/2013
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Plotting for Genset 1
if plot_G1 == 1
    % Preparing parameters for plotting
    p1_forPlot = zeros(size(s_G,3),1);
    q1_forPlot = zeros(size(s_G,3),1);
    for counter = 1:size(s_G,3)
        p1_forPlot(counter) = s_G(1,1,counter);
        q1_forPlot(counter) = s_G(1,2,counter);
    end
    p_loadG1 = P_load*CB_G_init(1)/(S_b_G(1)*N_G);
    q_loadG1 = Q_load*CB_G_init(1)/(S_b_G(1)*N_G);

    % Plotting
    figure()

    subplot(2,1,1)
    plot(time,p1_forPlot,time,p_loadG1)
    axis([0 runtime 0 2])
    h = ylabel('Active power [p.u.]');
    j = xlabel('Time [sec]');
    g = legend('$p_1$','$p_{load,1}$');
    set(h,'Interpreter','latex')
    set(j,'Interpreter','latex')
    set(g,'Interpreter','latex')
    grid on
    
    subplot(2,1,2)
    plot(time,q1_forPlot,time,q_loadG1)
    axis([0 runtime 0 2])
    h = ylabel('Reactive power [p.u.]');
    j = xlabel('Time [sec]');
    g = legend('$q_1$','$q_{load,1}$');
    set(h,'Interpreter','latex')
    set(j,'Interpreter','latex')
    set(g,'Interpreter','latex')
    grid on

    subplot(3,1,3)
    plot(time,abs_v_an)
    axis([0 runtime 0 2])
    h = ylabel('Bus voltage [p.u.]');
    j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    set(j,'Interpreter','latex')
    grid on
    SimulatedPower_G1 = gcf;

    % Comparing values simulated vs estimated

    figure()

    subplot(4,1,1)
    plot(time,delta_1)
    h = ylabel('$\delta_1$ [rad]');
    j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    set(j,'Interpreter','latex')
    grid on

    subplot(4,1,2)
    plot(time,omega_1)
    h = ylabel('$\omega_1$ [p.u.]');
    j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    set(j,'Interpreter','latex')
    grid on

    subplot(4,1,3)
    plot(time,t_m1)
    h = ylabel('$t_{m,1}$ [p.u.]');
    j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    set(j,'Interpreter','latex')
    grid on

    subplot(4,1,4)
    plot(time,i_f1)
    h = ylabel('$i_{f,1}$ [p.u.]');
    j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    set(j,'Interpreter','latex')
    grid on

    SimulatedStates_G1 = gcf;

    % Error values - Governor and AVR

    figure()
    suptitle('Genset 1: Error values - Governor and AVR')

    subplot(2,1,1)
    plot(omega_error1)
    %axis([0 runtime 0 2])
    h = ylabel('$\omega_{error,1}$ [p.u.]');
    j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    set(j,'Interpreter','latex')
    grid on

    subplot(2,1,2)
    plot(time,voltage_error1)
    %axis([0 runtime 0 2])
    xlabel('time [s]')
    ylabel('Voltage error (v_e_r_r_o_r) [p.u.]')
    legend('Voltage error (v_e_r_r_o_r) [p.u.]','voltage_error_BeforeFilter')
     h = ylabel('$\hat{v}_{an,error,1}$ [p.u.]');
    j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    set(j,'Interpreter','latex')
    grid on
    ErrorGovAVR_G1 = gcf;
end

%% Plotting for Genset 2
if plot_G2 == 1
    % Preparing parameters for plotting
    p2_forPlot = zeros(size(s_G,3),1);
    q2_forPlot = zeros(size(s_G,3),1);
    for i = 1:size(s_G,3)
        p2_forPlot(i) = s_G(2,1,i);
        q2_forPlot(i) = s_G(2,2,i);
    end
    
    p_loadG2 = P_load*CB_G_init(2)/(S_b_G(2)*N_G);
    q_loadG2 = Q_load*CB_G_init(2)/(S_b_G(2)*N_G);

    % Plotting
    figure()
    suptitle('Genset 2: Simulated values, power and voltage')

    subplot(2,1,1)
    plot(time,p2_forPlot,time,q2_forPlot,time,p_loadG2,time,q_loadG2)
    axis([0 runtime 0 2])
    xlabel('time [s]')
    ylabel('Power [p.u.]')
    legend('Active power (p_2)','Reactive power (q_2)','Desired load power (p_l_o_a_d_,_2)','Desired load power (q_l_o_a_d_,_2)')
    grid on

    subplot(2,1,2)
    plot(time,abs_v_an,time,voltage_ref1)
    axis([0 runtime 0 2])
    xlabel('time [s]')
    ylabel('v_a_n [p.u.]')
    legend('Bus voltage (v_a_n)','Desired voltage from AVR (v_r_e_f_,_2)')
    grid on
    SimulatedValues_PowerAndBusVoltage_G2 = gcf;

    % Genset 2: Comparing values simulated vs estimated

    figure()
    suptitle('Genset 2: Simulated values')

    subplot(5,1,1)
    plot(time,delta_2)
    xlabel('time [s]')
    ylabel('Power angle (\delta_2) [rad]')
    grid on

    subplot(5,1,2)
    plot(time,omega_2,time,omega_ref2)
    legend('\omega_2','\omega_r_e_f')
    xlabel('time [s]')
    ylabel('Frequency (\omega) [p.u.]')
    grid on

    subplot(5,1,3)
    plot(time,t_m2)
    xlabel('time [s]')
    ylabel('Mechanical torque (t_m_,_2) [p.u.]')
    grid on

    subplot(5,1,4)
    plot(time,i_f2)
    xlabel('time [s]')
    ylabel('Field current (i_f_,_2) [p.u.]')
    grid on

    subplot(5,1,5)
    plot(time,abs_e_a2)
    xlabel('time [s]')
    ylabel('Excitation voltage, rms value (e_a_,_2) [p.u.]')
    grid on
    SimulatedValues_AngFreqTorqueCurrentExVoltage_G2 = gcf;

    % Genset 2: Error values - Governor and AVR

    figure()
    suptitle('Genset 2: Error values - Governor and AVR')

    subplot(2,1,1)
    plot(omega_error2)
    axis([0 runtime 0 2])
    xlabel('time [s]')
    ylabel('Frequency error (\omega_e_r_r_o_r) [p.u.]')
    grid on

    subplot(2,1,2)
    plot(time,voltage_error2)
    axis([0 runtime 0 2])
    xlabel('time [s]')
    ylabel('Voltage error (v_e_r_r_o_r) [p.u.]')
    grid on
    ErrorValues_Governor_AVR_G2 = gcf;
end

%% Plotting for Genset 3
if plot_G3 == 1
    % Preparing parameters for plotting
    p3_forPlot = zeros(size(s_G,3),1);
    q3_forPlot = zeros(size(s_G,3),1);
    for i = 1:size(s_G,3)
        p3_forPlot(i) = s_G(3,1,i);
        q3_forPlot(i) = s_G(3,2,i);
    end
    
    p_loadG3 = P_load*CB_G_init(3)/(S_b_G(3)*N_G);
    q_loadG3 = Q_load*CB_G_init(3)/(S_b_G(3)*N_G);

    % Plotting
    figure()
    suptitle('Genset 3: Simulated values, power and voltage')

    subplot(2,1,1)
    plot(time,p3_forPlot,time,q3_forPlot,time,p_loadG3,time,q_loadG3)
    axis([0 runtime 0 2])
    xlabel('time [s]')
    ylabel('Power [p.u.]')
    legend('Active power (p_3)','Reactive power (q_3)','Desired load power (p_l_o_a_d_,_3)','Desired load power (q_l_o_a_d_,_3)')
    grid on

    subplot(2,1,2)
    plot(time,abs_v_an,time,voltage_ref3)
    axis([0 runtime 0 2])
    xlabel('time [s]')
    ylabel('v_a_n [p.u.]')
    legend('Bus voltage (v_a_n)','Desired voltage from AVR (v_r_e_f_,_3)')
    grid on
    SimulatedValues_PowerAndBusVoltage_G3 = gcf;

    % Genset 3: Comparing values simulated vs estimated

    figure()
    suptitle('Genset 3: Simulated values')

    subplot(5,1,1)
    plot(time,delta_3)
    xlabel('time [s]')
    ylabel('Power angle (\delta_3) [rad]')
    grid on

    subplot(5,1,2)
    plot(time,omega_3,time,omega_ref3)
    legend('\omega_3','\omega_r_e_f')
    xlabel('time [s]')
    ylabel('Frequency (\omega) [p.u.]')
    grid on

    subplot(5,1,3)
    plot(time,t_m3)
    xlabel('time [s]')
    ylabel('Mechanical torque (t_m_,_3) [p.u.]')
    grid on

    subplot(5,1,4)
    plot(time,i_f3)
    xlabel('time [s]')
    ylabel('Field current (i_f_,_3) [p.u.]')
    grid on

    subplot(5,1,5)
    plot(time,abs_e_a3)
    xlabel('time [s]')
    ylabel('Excitation voltage, rms value (e_a_,_3) [p.u.]')
    grid on
    SimulatedValues_AngFreqTorqueCurrentExVoltage_G3 = gcf;

    %% Genset 3: Error values - Governor and AVR

    figure()
    suptitle('Genset 3: Error values - Governor and AVR')

    subplot(2,1,1)
    plot(omega_error3)
    axis([0 runtime 0 2])
    xlabel('time [s]')
    ylabel('Frequency error (\omega_e_r_r_o_r) [p.u.]')
    grid on

    subplot(2,1,2)
    plot(time,voltage_error3)
    axis([0 runtime 0 2])
    xlabel('time [s]')
    ylabel('Voltage error (v_e_r_r_o_r) [p.u.]')
    grid on
    ErrorValues_Governor_AVR_G3 = gcf;
end