%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%% Initialize
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%--------------------------------------------------------------------------
%---- Usage ---------------------------------------------------------------
%--------------------------------------------------------------------------
%   Initializes program parameters
%--------------------------------------------------------------------------
%---- Inputs --------------------------------------------------------------
%--------------------------------------------------------------------------
%   No input
%--------------------------------------------------------------------------
%---- Outputs -------------------------------------------------------------
%--------------------------------------------------------------------------
%   Variable name       Dimension   Description  
%
%--------------------------------------------------------------------------
% Written by: Ida Bohne Krogseth
% Last Modified: 24/10/2012
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Base values
N = 2;
omega_b = 377;%[rad/s] 
omega_bm = omega_b*2/N;
V_b = 45e3;%[V]
S_b_G = [835e6 835e6 835e6]';%[W] 
Z_b = V_b^2./S_b_G;

%% Genset parameters
H_G = [5.6 5.6 5.6]';
D_fw_G = [0.03 0.03 0.03]';
tau_e_G = (0.9/omega_bm)*[1 1 1]';
AF_n_G = [30 30 30]';%[15 15 15]'; %[kg/kg]
AF_low_G = [14 14 14]';%[8 8 8]'; %[kg/kg]
AF_high_G = [23 23 23]';%[17 17 17]'; %[kg/kg]
m_a0_G = 0.9*[1 1 1]';%%AF_low_G./AF_high_G; %[-]  burde ke 0.8*[1 1 1]';%
T_t_G = [5 5 5]'; %[s]
r_f_G = [1 1 1]';
tau_f_G = tau_e_G;
z_G = [0.003 1.8;0.003 1.8;0.003 1.8];
Z_G = [z_G(1,:)*Z_b(1);z_G(2,:)*Z_b(2);z_G(3,:)*Z_b(3)];


%% Load
P_load1 = 3*0.6*S_b_G(1);
Q_load1 = 3*0.6*S_b_G(1);

P_load2 = 3*1*S_b_G(1);
Q_load2 = 3*0.5*S_b_G(1);

LoadStepTime = 200;

if CaseNr == 0
    LoadStepTime = 200;
    P_load1 = 1*3*S_b_G(1);
    Q_load1 = 3*0.5*S_b_G(1);

    P_load2 = 1*3*S_b_G(1);
    Q_load2 = 3*0.5*S_b_G(1); 
elseif CaseNr == 12 || CaseNr == 13 || CaseNr == 14
    LoadStepTime = 200;
    P_load1 = 0.6*3*S_b_G(1);
    Q_load1 = 0.5*S_b_G(1);

    P_load2 = 0.6*3*S_b_G(1);
    Q_load2 = 0.5*S_b_G(1);
elseif CaseNr == 22 || CaseNr == 23 || CaseNr == 24
    LoadStepTime = 200;
    P_load1 = 0.6*3*S_b_G(1);
    Q_load1 = 0.5*S_b_G(1);

    P_load2 = 0.6*3*S_b_G(1);
    Q_load2 = 0.5*S_b_G(1);
elseif CaseNr == 31 || CaseNr == 32 || CaseNr == 33
    LoadStepTime = 200;
    P_load1 = 0.6*3*S_b_G(1);
    Q_load1 = 0.5*S_b_G(1);

    P_load2 = 0.6*3*S_b_G(1);
    Q_load2 = 0.5*S_b_G(1);
elseif CaseNr == 41
    LoadStepTime = 20;
    P_load1 = 0.01*3*S_b_G(1);
    Q_load1 = 3*0.5*S_b_G(1);

    P_load2 = 1*3*S_b_G(1);
    Q_load2 = 3*0.5*S_b_G(1);    
elseif CaseNr == 42
    LoadStepTime = 20;
    P_load1 = 1*3*S_b_G(1);
    Q_load1 = 3*0.5*S_b_G(1);

    P_load2 = 0.01*3*S_b_G(1);
    Q_load2 = 3*0.5*S_b_G(1);
elseif CaseNr == 51
    LoadStepTime = 20;
    P_load1 = 1*3*S_b_G(1);
    Q_load1 = 3*0.01*S_b_G(1);

    P_load2 = 1*3*S_b_G(1);
    Q_load2 = 3*1*S_b_G(1);
elseif CaseNr == 52
    LoadStepTime = 20;
    P_load1 = 1*3*S_b_G(1);
    Q_load1 = 3*1*S_b_G(1);

    P_load2 = 1*3*S_b_G(1);
    Q_load2 = 3*0.01*S_b_G(1);
end
%% Governor and AVR (K_speed and K_voltage are the controller gains; G1: K(1,:), G2: K(2,:), G3: K(3,:)
droop_speed = [0.05 0.05 0.05]';
droop_voltage = [0.05 0.05 0.05]';

K_Governor = [100 30 0;100 30 0;100 30 0]; %K(1,:)=[P-gain I-gain D-gain]
K_AVR = [10 5 0;10 5 0;10 5 0]; %K(1,:)=[P-gain I-gain D-gain] 0.8 0.5 0

InitialValueAVR = [0 0 0]';%1.05
AvrSamp = [0.01 0.01 0.01]';

v_noload = [3 3 3]';%[1 1 1]'+1*droop_voltage;%
omega_noload = [1.7 1.7 1.7]';%[1 1 1]'+1*droop_speed;%

Governor_limits = [0.1 1.25;0.1 1.25;0.1 1.25]; %Lower and upper fuel rack position (u) 2
AVR_limits = [0.01 10;0.01 10;0.01 10]; %Lower and upper field voltage (v_f) 20

omega_maxload = omega_noload - droop_speed*2;
v_maxload = v_noload - droop_voltage*2;

%% Liniarization values
x_0 = [0 omega_noload(1) 1 1]'; % [delta0, omega0, t_m0, i_f0]'
u_0 = [omega_noload(1) v_noload(1) 1 2]'; % [omega_00, abs_v_ll0, u0, v_f0]'
f_0 = [0 0]';

%% Simulation conditions 
runtime = 50;
CB_G_init = [1 1 1]; % Generator circuit breaker status [G1 G2 G3]' before DisconnectionTime_Gensets
CB_G_init2 = [1 1 1]; % Generator circuit breaker status [G1 G2 G3]' after DisconnectionTime_Gensets
N_G = CB_G_init(1)+CB_G_init(2)+CB_G_init(3);
DisconnectionTime_Gensets = [200 200 200]';

if CaseNr == 31
    DisconnectionTime_Gensets = [20 20 20]';
    CB_G_init2(1) = 0;
elseif CaseNr == 32
    DisconnectionTime_Gensets = [20 20 20]';
    CB_G_init2(2) = 0;
elseif CaseNr == 33
    DisconnectionTime_Gensets = [20 20 20]';
    CB_G_init2(3) = 0;
end

%% Failure modes
% Governor
f_u_matrix = [0 0 0 0;0 0 0 0;0 0 0 0]; % [Normal operation, Gov frozen, Gov to min, Gov to max]
GovernorFaultTime = [200 200 200]';
if CaseNr == 12
    f_u_matrix(1,2) = 1;
    GovernorFaultTime = [20 20 20]';
elseif CaseNr == 13
    f_u_matrix(1,3) = 1;
    GovernorFaultTime = [20 20 20]';
elseif CaseNr == 14
    f_u_matrix(1,4) = 1;
    GovernorFaultTime = [20 20 20]';
end

f_u_magnitudes =   [0.5 0 Governor_limits(1,2); 
                    0.5 0 Governor_limits(2,2);
                    0.5 0 Governor_limits(3,2)];% [Frozen value, min value, max value]

% AVR                
f_vf_matrix = [0 0 0 0;0 0 0 0;0 0 0 0]; % [Normal operation, AVR frozen, AVR to min, AVR to max]
AVRFaultTime = [200 200 200]';
if CaseNr == 22
    f_vf_matrix(1,2) = 1;
    AVRFaultTime = [20 20 20]';
elseif CaseNr == 23
    f_vf_matrix(1,3) = 1;
    AVRFaultTime = [20 20 20]';
elseif CaseNr == 24
    f_vf_matrix(1,4) = 1;
    AVRFaultTime = [20 20 20]';
end

f_vf_magnitudes =   [1.9 1.799 2; 
                    1.9 1.799 2;
                    1.9 1.799 2];% [Frozen value, min value, max value]

%% Disconnection limits
DisconnectWhenFaultDetected = 0;
FaultDetectionInitTime = 20+0.002; % [sec]
diff_r = (1+0.01)*[-873.1 -546.2;1.515 19.75;2.277*10^5 3.652*10^5];   %[diff_rdq_min diff_rdq_max;diff_rddw_min diff_rddw_max;diff_rddq_min diff_rddq_max]



