%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%% Plotting
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%--------------------------------------------------------------------------
%---- Usage ---------------------------------------------------------------
%--------------------------------------------------------------------------
%   Plots simulated values from SimulationPlantModel
%--------------------------------------------------------------------------
%---- Inputs --------------------------------------------------------------
%--------------------------------------------------------------------------
%   No input
%--------------------------------------------------------------------------
%---- Outputs -------------------------------------------------------------
%--------------------------------------------------------------------------
%   Variable name       Dimension   Description  
%
%--------------------------------------------------------------------------
% Written by: Ida Bohne Krogseth
% Last Modified: 23/04/2013
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Plotting for Genset 1

if plot_NonlinRR == 1
    set(0,'DefaultAxesFontSize',10)
    figure()
    hold on
    
    subplot(4,1,1)
    plot(time,r_dq1,time,r_dq2,time,r_dq3)
%     if CaseNr == 0 || CaseNr == 14
%         axis([0 runtime -440 -320])
%     elseif CaseNr == 33
%         axis([0 runtime -0.5*10^12 3*10^12])
%     elseif CaseNr == 41
%         axis([0 runtime -0.5*10^11 3.5*10^11])
%     end
    h = ylabel('$RR \dot{q}$');
    %j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    %set(j,'Interpreter','latex')
    grid off
    
    subplot(4,1,2)
    plot(time,r_ddw1,time,r_ddw2,time,r_ddw3)
%     if CaseNr == 0 || CaseNr == 14
%         axis([0 runtime -100 100])
%     elseif CaseNr == 33
%         axis([0 runtime -0.6*10^12 0.3*10^12])
%     end
    h = ylabel('$RR \ddot{\omega}$');
    %j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    %set(j,'Interpreter','latex')
    grid off

    subplot(4,1,3)
    plot(time,r_ddq1,time,r_ddq2,time,r_ddq3)
%     if CaseNr == 0 || CaseNr == 14
%         axis([0 runtime 1*10^5 2*10^5])
%     elseif CaseNr == 33
%         axis([0 runtime -15*10^24 15*10^24])
%     end
    h = ylabel('$RR \ddot{q}$');
    j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    set(j,'Interpreter','latex')
    grid off
    
    legendAll =subplot(4,1,4);
    plot(0,0,0,0,0,0)
    set(legendAll,'Visible','off')%'vis','on'
    g = legend('Genset 1 $\quad$','  Genset 2 $\quad$','  Genset 3 $\quad$');
    set(g,'Interpreter','latex')
    set(g,'Location','NorthOutside')
    set(g,'Orientation','horizontal')
    
    
    NonlinRRValues = gcf;
    hold off
    
    % Plotting state error
    figure()
    hold on

    subplot(5,1,1)
    plot(time,delta_1-x11,time,delta_2-x12,time,delta_3-x13)
%      if PlotLowGov == 1
%         axis([0 runtime -1*10^(-16) 1*10^(-16)])
%      end
    h = ylabel('$\tilde{\delta}$ [rad]');
    %h = ylabel('Power angle [rad]');
    %j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    %set(j,'Interpreter','latex')
    grid off

    subplot(5,1,2)
    plot(time,omega_1-x21,time,omega_2-x22,time,omega_3-x23)
%      if PlotLowGov == 1
%         axis([0 runtime -1*10^(-16) 1*10^(-16)])
%      end
    %h = ylabel('Frequency [p.u.]');
    h = ylabel('$\tilde{\omega}$ [p.u.]');
    %j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    %set(j,'Interpreter','latex')
    grid off

    subplot(5,1,3)
    plot(time,t_m1-x31,time,t_m2-x32,time,t_m3-x33)
%     if CaseNr == 14
%         axis([0 runtime -0.1 1.6])
%     elseif PlotLowGov == 1
%         axis([0 runtime -0.01 0.01])
%     end
    %h = ylabel('Mechanical torque [p.u.]');
    h = ylabel('$\tilde{t}_m$ [p.u.]');
    %j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    %set(j,'Interpreter','latex')
    grid off

    subplot(5,1,4)
    plot(time,i_f1-x41,time,i_f2-x42,time,i_f3-x43)
%      if PlotLowAVR == 1
%         axis([0 runtime -1*10^(-15) 1*10^(-15)])
%      end
    % h = ylabel('Field current [p.u.]');
    h = ylabel('$\tilde{i}_f$ [p.u.]');
    j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    set(j,'Interpreter','latex')
    grid off
    
    legendAll =subplot(5,1,5);
    plot(0,0,0,0,0,0)
    set(legendAll,'Visible','off')%'vis','on'
    g = legend('Genset 1 $\quad$','  Genset 2 $\quad$','  Genset 3 $\quad$');
    set(g,'Interpreter','latex')
    set(g,'Location','NorthOutside')
    set(g,'Orientation','horizontal')
    
    StateEstimationError = gcf;
    hold off

    %% Differentiation values
    
    figure()
    hold on

    subplot(3,3,1)
    plot(time,y11)
    h = ylabel('$\omega_1$ [p.u.]');
    %j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    %set(j,'Interpreter','latex')
    grid off
    
    subplot(3,3,4)
    plot(time,y21)
    h = ylabel('$p_1$ [p.u.]');
    %j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    %set(j,'Interpreter','latex')
    grid off
    
    subplot(3,3,7)
    plot(time,y31)
    h = ylabel('$q_1$ [p.u.]');
    j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    set(j,'Interpreter','latex')
    grid off
    
    subplot(3,3,2)
    plot(time,d_y11)
    h = ylabel('$\dot{\omega}_1$ [p.u.]');
    %j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    %set(j,'Interpreter','latex')
    grid off
    
    subplot(3,3,5)
    plot(time,d_y21)
    h = ylabel('$\dot{p}_1$ [p.u.]');
    j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    set(j,'Interpreter','latex')
    grid off
    
    subplot(3,3,8)
    plot(time,d_y31)
    h = ylabel('$\dot{q}_1$ [p.u.]');
    j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    set(j,'Interpreter','latex')
    grid off
    
    subplot(3,3,3)
    plot(time,dd_y11)
    h = ylabel('$\ddot{\omega}_1$ [p.u.]');
    %j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    %set(j,'Interpreter','latex')
    grid off
    
    subplot(3,3,6)
    plot(time,dd_y21)
    h = ylabel('$\ddot{p}_1$ [p.u.]');
    %j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    %set(j,'Interpreter','latex')
    grid off
    
    subplot(3,3,9)
    plot(time,dd_y31)
    h = ylabel('$\ddot{q}_1$ [p.u.]');
    j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    set(j,'Interpreter','latex')
    grid off
    
    DiffValuesY = gcf;
    hold off
    
    % u
    
    figure()
    hold on

    subplot(4,3,1)
    plot(time,u11)
    h = ylabel('$\omega_0$ [p.u.]');
    %j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    %set(j,'Interpreter','latex')
    grid off
    
    subplot(4,3,2)
    plot(time,d_u11)
    h = ylabel('$\dot{\omega}_0$ [p.u.]');
    %j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    %set(j,'Interpreter','latex')
    grid off
    
    subplot(4,3,3)
    plot(time,dd_u11)
    h = ylabel('$\ddot{\omega}_0$ [p.u.]');
    %j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    %set(j,'Interpreter','latex')
    grid off
    
    subplot(4,3,4)
    plot(time,u21)
    h = ylabel('$\hat{v}_{ll}$ [p.u.]');
    %j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    %set(j,'Interpreter','latex')
    grid off
    
    subplot(4,3,5)
    plot(time,d_u21)
    h = ylabel('$\dot{\hat{v}}_{ll}$ [p.u.]');
    %j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    %set(j,'Interpreter','latex')
    grid off
    
    subplot(4,3,6)
    plot(time,dd_u21)
    h = ylabel('$\ddot{\hat{v}}_{ll}$ [p.u.]');
    %j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    %set(j,'Interpreter','latex')
    grid off
    
    subplot(4,3,7)
    plot(time,u31)
    h = ylabel('$u_1$ [p.u.]');
    %j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    %set(j,'Interpreter','latex')
    grid off
    
    subplot(4,3,8)
    plot(time,d_u31)
    h = ylabel('$\dot{u}_1$ [p.u.]');
    %j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    %set(j,'Interpreter','latex')
    grid off
    
    subplot(4,3,9)
    plot(time,dd_u31)
    h = ylabel('$\ddot{u}_1$ [p.u.]');
    %j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    %set(j,'Interpreter','latex')
    grid off
    
    subplot(4,3,10)
    plot(time,u41)
    h = ylabel('$v_{f,1}$ [p.u.]');
    j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    set(j,'Interpreter','latex')
    grid off
    
    subplot(4,3,11)
    plot(time,d_u41)
    h = ylabel('$\dot{v}_{f,1}$ [p.u.]');
    j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    set(j,'Interpreter','latex')
    grid off
    
    subplot(4,3,12)
    plot(time,dd_u41)
    h = ylabel('$\ddot{v}_{f,1}$ [p.u.]');
    j = xlabel('Time [sec]');
    set(h,'Interpreter','latex')
    set(j,'Interpreter','latex')
    grid off
    
    DiffValuesU = gcf;
    hold off
end

